package com.famoco.smartcardexample;

import android.app.Activity;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.EditText;
import android.widget.Spinner;
import android.widget.TextView;
import android.widget.Toast;

import com.famoco.secommunication.ALPARProtocol;
import com.famoco.secommunication.SmartcardReader;
import com.famoco.secommunication.UnsuccessfulCommandException;
import com.famoco.setest.R;
import com.famoco.utils.ByteArrayUtils;

public class SmartcardReaderExampleActivity extends Activity {
	private static final String TAG = "SmartcardReaderExample";
	
	private SmartcardReader mSmartcardReader;

	// APDU constants
	private final static String SELECTION_NONE = "Custom";
	
	private final static String COMMAND_SAM_GET_VERSION = "SAM_GetVersion";
	private final static String APDU_SAM_GET_VERSION = "80 60 00 00 00";
	
	private final static String COMMAND_SELECT_CARD_MANAGER = "SELECT Card Manager";
	private final static String APDU_SELECT_CARD_MANAGER = "00 A4 04 00 00";
	
	private final static String[] COMMAND_ARRAY = new String[] {
		COMMAND_SELECT_CARD_MANAGER,
		COMMAND_SAM_GET_VERSION,
		SELECTION_NONE,
	};
	
	private final static String[] APDU_ARRAY = new String[] {
		APDU_SELECT_CARD_MANAGER,
		APDU_SAM_GET_VERSION,
		"",
	};	
	
	// Card clock constants
	private final static String CARD_CLOCK_FREQUENCY_14_745MHz = "14.745MHz";
	private final static String CARD_CLOCK_FREQUENCY_7_37MHz  = "7.37MHz";
	private final static String CARD_CLOCK_FREQUENCY_3_68MHz  = "3.68MHz (Default)";
	private final static String CARD_CLOCK_FREQUENCY_1_84MHz  = "1.84MHz";

	private final static String[] CARD_CLOCK_ARRAY = new String[] {
		CARD_CLOCK_FREQUENCY_1_84MHz,
		CARD_CLOCK_FREQUENCY_3_68MHz,
		CARD_CLOCK_FREQUENCY_7_37MHz,
		CARD_CLOCK_FREQUENCY_14_745MHz,		
	};
	
	private final static byte[] CARD_CLOCK_VALUE_ARRAY = new byte[] {
		ALPARProtocol.PARAM_CLOCK_FREQUENCY_1_84MHz,
		ALPARProtocol.PARAM_CLOCK_FREQUENCY_3_68MHz,
		ALPARProtocol.PARAM_CLOCK_FREQUENCY_7_37MHz,
		ALPARProtocol.PARAM_CLOCK_FREQUENCY_14_745MHz,
	};
	
	
	private Button mRunButton;
	
	private Button mResetButton;
	
	private TextView mAPIVersionTextView;
	
	private TextView mFirmwareVersionTextView;
	
	private TextView mATRTextView;
	
	private CheckBox mEnableDebuggingCheckBox;
	
    private CheckBox mAutoNegotiateCheckBox;
    
    private Spinner mCommandSpinner;
    
    private Spinner mCardClockSpinner;
    
    private EditText mApduEditText;
    
    private TextView mRespTextView;
    
    private TextView mTimeTextView;
    
	
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_main);

		// obtain smartcard reader instance 
		mSmartcardReader = SmartcardReader.getInstance();
		
		mEnableDebuggingCheckBox = (CheckBox) findViewById(R.id.enable_debugging);
		mAutoNegotiateCheckBox = (CheckBox) findViewById(R.id.auto_negotiate);
		
        // baud rate setup
        mCommandSpinner = (Spinner) findViewById(R.id.command_spinner);
        ArrayAdapter<String> commandAdapter = new ArrayAdapter<String>(this, android.R.layout.simple_spinner_item, COMMAND_ARRAY);
        mCommandSpinner.setAdapter(commandAdapter);
        mCommandSpinner.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
        	public void onItemSelected(android.widget.AdapterView<?> parent, View view, int pos, long id) {
        		String apdu = APDU_ARRAY[pos];
    			mApduEditText.setText(apdu);
        	}
        	
        	public void onNothingSelected(android.widget.AdapterView<?> parent) {
        	}
		});
        
        mCardClockSpinner = (Spinner) findViewById(R.id.card_clock);
        ArrayAdapter<String> cardClockAdapter = new ArrayAdapter<String>(this, android.R.layout.simple_spinner_item, CARD_CLOCK_ARRAY);
        mCardClockSpinner.setAdapter(cardClockAdapter);
        mCardClockSpinner.setSelection(1); // default is 3.68MHz
        
        mApduEditText = (EditText) findViewById(R.id.apdu);
        
		mRunButton = (Button) findViewById(R.id.run);
		mRunButton.setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View paramView) {
				String hexString = mApduEditText.getEditableText().toString();
				byte[] apdu = null;
				try {
					apdu = ByteArrayUtils.fromHexString(hexString);
				} catch (Exception e) {
					Log.d(TAG, e.getLocalizedMessage(), e);
					Toast.makeText(SmartcardReaderExampleActivity.this, "Invalid APDU", Toast.LENGTH_SHORT).show();
					return;
				}
				
				try {
					run(apdu);
				} catch (Exception e) {
					Toast.makeText(SmartcardReaderExampleActivity.this, "ERROR: " + e.getMessage(), Toast.LENGTH_SHORT).show();
				}
			}
		});
		
		mResetButton = (Button) findViewById(R.id.reset);
		mResetButton.setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View paramView) {
				reset();
			}
		});	
		
		mAPIVersionTextView = (TextView) findViewById(R.id.api_version);
		mAPIVersionTextView.setText(SmartcardReader.VERSION);
		
		mFirmwareVersionTextView = (TextView) findViewById(R.id.firmware_version);
		
		mATRTextView = (TextView) findViewById(R.id.atr);
		
		mRespTextView = (TextView) findViewById(R.id.resp);
		
		mTimeTextView = (TextView) findViewById(R.id.time);
	}
	
	@Override
	protected void onResume() {
		super.onResume();
		
	}
	
	private void reset() {
		mEnableDebuggingCheckBox.setChecked(true);
		mAutoNegotiateCheckBox.setChecked(true);
		mCommandSpinner.setSelection(0);
		mApduEditText.setText(APDU_ARRAY[0]);
		
		mCardClockSpinner.setSelection(1);
		
		mFirmwareVersionTextView.setText(R.string.not_available);
		mATRTextView.setText(R.string.not_available);
		mRespTextView.setText(R.string.not_available);
		mTimeTextView.setText(R.string.not_available);
	}
	
	private void initSmartcard() {
		// debugging enabled, it should be set to false in production code.
		SmartcardReader.setDebuggingEnabled(mEnableDebuggingCheckBox.isChecked());
		
		// open smartcard reader.
		mSmartcardReader.openReader(115200);
		
		String firmwareVersion = mSmartcardReader.getFirmwareVersion();
		Log.d(TAG, "firmwareVersion = " + firmwareVersion);
		mFirmwareVersionTextView.setText(firmwareVersion);
		
		// auto negotiate for optimal speed.
		mSmartcardReader.setAutoNegotiate(mAutoNegotiateCheckBox.isChecked());
		
		// power on smartcard reader
		byte[] atr = mSmartcardReader.powerOn();
		
		int pos = mCardClockSpinner.getSelectedItemPosition();
		byte frequency = CARD_CLOCK_VALUE_ARRAY[pos]; 
		mSmartcardReader.setClockCard(frequency);
		
		String atrHex = ByteArrayUtils.bytesToHexString(atr, true);
		Log.d(TAG, "ATR = " + atrHex);
		mATRTextView.setText(atrHex);
	}
	
	private void deinitSmartcard() {
		// power off smartcard reader
		mSmartcardReader.powerOff();
		
		// close smartcard reader
		mSmartcardReader.closeReader();
	}
	
	private void sendApdu(byte[] apdu) {
		// send APDU
		Log.d(TAG, "APDU => " + ByteArrayUtils.bytesToHexString(apdu));
		long start = System.currentTimeMillis();
		byte[] resp = mSmartcardReader.sendApdu(apdu);
		long end = System.currentTimeMillis();
		String s = ByteArrayUtils.bytesToHexString(resp, true);
		Log.d(TAG, "APDU <= " + s);
		
		mRespTextView.setText(s);
		mTimeTextView.setText(String.format("%dms", (end - start)));
	}
	
	private void run(byte[] apdu) {
		try {
			initSmartcard();
			sendApdu(apdu);
		} catch (UnsuccessfulCommandException e) {
			throw e;
		} finally {
			deinitSmartcard();
		}
	}
	
	@Override
	protected void onPause() {
		super.onPause();
	}
	
	@Override
	protected void onDestroy() {
		super.onDestroy();
	}
	
}
